package com.zsc.edu.dify.modules.dify.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.zsc.edu.dify.exception.ExceptionUtil;
import com.zsc.edu.dify.framework.mybatisplus.DataPermission;
import com.zsc.edu.dify.framework.security.SecurityUtil;
import com.zsc.edu.dify.modules.dify.entity.WorkflowData;
import com.zsc.edu.dify.modules.dify.service.AppEntityService;
import com.zsc.edu.dify.modules.dify.service.DifyWorkflowService;
import com.zsc.edu.dify.modules.operationLog.entity.OperationLogAnnotation;
import io.github.guoshiqiufeng.dify.core.pojo.DifyPageResult;
import io.github.guoshiqiufeng.dify.workflow.DifyWorkflow;
import io.github.guoshiqiufeng.dify.workflow.dto.request.WorkflowLogsRequest;
import io.github.guoshiqiufeng.dify.workflow.dto.request.WorkflowRunRequest;
import io.github.guoshiqiufeng.dify.workflow.dto.response.*;
import jakarta.annotation.Resource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;

import java.util.List;

@RestController
@RequestMapping("/api/v1/workflow")
public class V1WorkflowController {

    @Resource
    private DifyWorkflow difyWorkflow;

    @Resource
    private AppEntityService appEntityService;

    @Resource
    private DifyWorkflowService  difyWorkflowService;

    /**
     * 运行工作流
     *
     * @param request
     * @return
     */
    @PostMapping("/run/{appId}")
    @OperationLogAnnotation(content = "'dify工作流'", operationType = "运行")
    public WorkflowRunResponse runWorkflow(@RequestBody WorkflowRunRequest request, @PathVariable String appId) {
        request.setUserId(SecurityUtil.getUserInfo().id.toString());
        return ExceptionUtil.difyException(() -> difyWorkflowService.run(request, appId));
    }

    /**
     * 运行工作流(流式)
     *
     * @param request
     * @return
     */
    @PostMapping("/run/stream")
    public Flux<WorkflowRunStreamResponse> runWorkflowStream(@RequestBody WorkflowRunRequest request) {
        return difyWorkflow.runWorkflowStream(request);
    }

    /**
     * 停止工作流(流式)
     *
     * @param taskId
     * @return
     */
    @PatchMapping("/stop/{appId}")
    public WorkflowStopResponse stopWorkflowStream(String taskId, @PathVariable String appId) {
        String apiKey =appEntityService.getApikey(appId);
        String userId = SecurityUtil.getUserInfo().id.toString();
        return difyWorkflow.stopWorkflowStream(apiKey, taskId,userId);
    }

    /**
     * 获取工作流信息
     *
     * @param workflowRunId
     * @return
     */
    @GetMapping("/info/{appId}")
    public WorkflowInfoResponse info(String workflowRunId, @PathVariable String appId) {
        String apiKey =appEntityService.getApikey(appId);
        return ExceptionUtil.difyException(() -> difyWorkflow.info(workflowRunId, apiKey));
    }

    /**
     * 获取工作流日志
     *
     * @param request
     * @return
     */
    @PostMapping("/logs/{appId}")
    public DifyPageResult<WorkflowLogs> logs(@RequestBody WorkflowLogsRequest request, @PathVariable String appId) {
        String apiKey = appEntityService.getApikey(appId);
        request.setApiKey(apiKey);
        final DifyPageResult<WorkflowLogs> logs = difyWorkflow.logs(request);
        return logs;
    }

    /**
     * 日志列表
     *
     * @param appId
     * @return
     */
    @GetMapping("/list/{appId}")
    @DataPermission
    public List<WorkflowData> list(@PathVariable String appId){
        return difyWorkflowService.list(new QueryWrapper<WorkflowData>().eq("app_id",appId));
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @GetMapping("/detail/{id}")
    @DataPermission
    public WorkflowData detail(@PathVariable Long id){
        return difyWorkflowService.detail(id);
    }

    /**
     * 删除
     *
     * @param id
     * @return
     */
    @DeleteMapping("/delete/{id}")
    @OperationLogAnnotation(content = "'dify工作流日志'", operationType = "删除")
    public boolean delete(@PathVariable Long id){
        return difyWorkflowService.delete(id);
    }

    /**
     * 分页查询
     *
     * @param page
     * @return
     */
    @GetMapping("/query")
    @DataPermission
    public Page<WorkflowData> query(Page<WorkflowData> page){
        return difyWorkflowService.query(page);
    }
}
