package com.zsc.edu.dify.modules.dify.controller;

import com.zsc.edu.dify.exception.ExceptionUtil;
import com.zsc.edu.dify.framework.mybatisplus.DataPermission;
import com.zsc.edu.dify.framework.security.SecurityUtil;
import com.zsc.edu.dify.modules.dify.entity.AppEntity;
import com.zsc.edu.dify.modules.dify.service.AppEntityService;
import com.zsc.edu.dify.modules.operationLog.entity.OperationLogAnnotation;
import io.github.guoshiqiufeng.dify.chat.DifyChat;
import io.github.guoshiqiufeng.dify.chat.dto.request.ChatMessageSendRequest;
import io.github.guoshiqiufeng.dify.chat.dto.response.ChatMessageSendResponse;
import jakarta.annotation.Resource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/api/word")
public class WordController {

    @Resource
    private DifyChat difyChat;
    @Resource
    private AppEntityService appEntityService;

    /**
     * 发送职业创新申报书_对话流路线
     *
     * @param sendRequest 消息参数 （可以自定义参数，调用 difyChat 实例时重新组装即可），
     *                    用户 id可以改为从上下文（token）获取，
     *                    apikey 建议在数据库进行存储，前端调用时传智能体 id，从数据库查询
     */
    @PostMapping("/completions")
    @OperationLogAnnotation(content = "'dify对话'", operationType = "发送")
    public ChatMessageSendResponse sendChatMessage(@RequestBody ChatMessageSendRequest sendRequest){
        sendRequest.setApiKey(appEntityService.getApikey("baca08c1-e92b-4dc9-a445-3584803f54d4"));
        sendRequest.setUserId(SecurityUtil.getUserInfo().id.toString());
        return ExceptionUtil.difyException(()->difyChat.send(sendRequest));
    }

    /**
     * 根据appType获取应用列表
     * @return
     */
    @GetMapping("/apps")
    @DataPermission
    public List<AppEntity> getAppsByAppType(){
        return appEntityService.selectByAppType(AppEntity.AppType.WORD.getValue());
    }
}
